package com.yaoyicloud.easier.common.core.constant;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

/**
 * 缓存常量
 *
 * <p>
 * 全局缓存，在缓存名称上加上该前缀表示该缓存不区分租户，比如:
 * {@code @Cacheable(value = CacheConstants.GLOBALLY+CacheConstants.MENU_DETAILS, key = "#roleId  + '_menu'", unless = "#result == null")}
 * </p>
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-12-19 11:20
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class CacheConstants {

    /**
     * 全局缓存，在缓存名称上加上该前缀表示该缓存不区分租户
     */
    public static final String GLOBALLY = "gl:";

    /**
     * 验证码前缀
     */
    public static final String DEFAULT_CODE_KEY = "DEFAULT_CODE_KEY:";

    /**
     * 菜单信息缓存
     */
    public static final String MENU_DETAILS = "menu_details";

    /**
     * 移动端菜单信息缓存
     */
    public static final String APP_MENU_DETAILS = "app_menu_details";

    /**
     * 用户信息缓存
     */
    public static final String USER_DETAILS = "user_details";

    /**
     * 移动端用户信息缓存
     */
    public static final String MEMBER_DETAILS = "member_details";

    /**
     * 角色信息缓存
     */
    public static final String ROLE_DETAILS = "role_details";

    /**
     * 字典信息缓存
     */
    public static final String DICT_DETAILS = "dict_details";

    /**
     * 商品分类标签
     */
    public static final String MALL_CATE_DETAILS = "mall_cate_details";

    /**
     * 商品规格
     */
    public static final String MALL_PROD_SPEC_DETAILS = "mall_prod_spec_details";

    /**
     * 商品详情
     */
    public static final String MALL_PROD_DETAILS = "mall_prod_details";

    /**
     * 项目标签详情
     */
    public static final String PROJ_TAG_DETAILS = "proj_tag_details";

    /**
     * oauth 客户端信息
     */
    public static final String CLIENT_DETAILS_KEY = "easier_oauth:client:details";

    /**
     * spring boot admin 事件key
     */
    public static final String EVENT_KEY = GLOBALLY + "event_key";

    /**
     * 路由存放
     */
    public static final String ROUTE_KEY = GLOBALLY + "gateway_route_key";

    /**
     * 内存reload 时间
     */
    public static final String ROUTE_JVM_RELOAD_TOPIC = "gateway_jvm_route_reload_topic";

    /**
     * redis 重新加载 路由信息
     */
    public static final String ROUTE_REDIS_RELOAD_TOPIC = "upms_redis_route_reload_topic";

    /**
     * redis 重新加载客户端信息
     */
    public static final String CLIENT_REDIS_RELOAD_TOPIC = "upms_redis_client_reload_topic";

    /**
     * 公众号 reload
     */
    public static final String MP_REDIS_RELOAD_TOPIC = "mp_redis_reload_topic";

    /**
     * 支付 reload 事件
     */
    public static final String PAY_REDIS_RELOAD_TOPIC = "pay_redis_reload_topic";

    /**
     * 参数缓存
     */
    public static final String PARAMS_DETAILS = "params_details";

    /**
     * 租户缓存 (不区分租户)
     */
    public static final String TENANT_DETAILS = GLOBALLY + "tenant_details";

    /**
     * i18n缓存 (不区分租户)
     */
    public static final String I18N_DETAILS = GLOBALLY + "i18n_details";

    /**
     * 客户端配置缓存
     */
    public static final String CLIENT_FLAG = "client_config_flag";

    /**
     * 登录错误次数
     */
    public static final String LOGIN_ERROR_TIMES = "login_error_times";

    /**
     * oauth 缓存前缀
     */
    public static final String PROJECT_OAUTH_ACCESS = "token::access_token";

}
