package com.yaoyicloud.easier.common.core.constant;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

/**
 * 安全常量
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-12-19 11:22
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class SecurityConstants {

    /**
     * 启动时是否检查Inner注解安全性
     */
    public static final boolean INNER_CHECK = true;

    /**
     * 刷新
     */
    public static final String REFRESH_TOKEN = "refresh_token";

    /**
     * 验证码有效期
     */
    public static final int CODE_TIME = 60;

    /**
     * 验证码长度
     */
    public static final String CODE_SIZE = "6";

    /**
     * 角色前缀
     */
    public static final String ROLE = "ROLE_";

    /**
     * 前缀
     */
    public static final String EASIER_PREFIX = "easier_";

    /**
     * token 相关前缀
     */
    public static final String TOKEN_PREFIX = "token:";

    /**
     * oauth 相关前缀
     */
    public static final String OAUTH_PREFIX = "oauth:";

    /**
     * 授权码模式code key 前缀
     */
    public static final String OAUTH_CODE_PREFIX = "oauth:code:";

    /**
     * 项目的license
     */
    public static final String EASIER_LICENSE = "https://easier.yaoyicloud.com";

    /**
     * 内部
     */
    public static final String FROM_IN = "Y";

    /**
     * 标志
     */
    public static final String FROM = "from";

    /**
     * 请求header
     */
    public static final String HEADER_FROM_IN = FROM + "=" + FROM_IN;

    /**
     * OAUTH URL
     */
    public static final String OAUTH_TOKEN_URL = "/oauth2/token";

    /**
     * 移动端授权
     */
    public static final String GRANT_MOBILE = "mobile";

    /**
     * 客户类型
     */
    public static final String CLIENT_TYPE_HEADER = "X-Client-Type";

    /**
     * QQ获取token
     */
    public static final String QQ_AUTHORIZATION_CODE_URL = "https://graph.qq.com/oauth2.0/token?grant_type="
        + "authorization_code&code=%S&client_id=%s&redirect_uri=" + "%s&client_secret=%s";

    /**
     * 微信获取OPENID
     */
    public static final String WX_AUTHORIZATION_CODE_URL = "https://api.weixin.qq.com/sns/oauth2/access_token"
        + "?appid=%s&secret=%s&code=%s&grant_type=authorization_code";

    /**
     * 微信小程序OPENID
     */
    public static final String MINI_APP_AUTHORIZATION_CODE_URL =
        "https://api.weixin.qq.com/sns/jscode2session" + "?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code";

    /**
     * 码云获取token
     */
    public static final String GITEE_AUTHORIZATION_CODE_URL = "https://gitee.com/oauth/token?grant_type="
        + "authorization_code&code=%S&client_id=%s&redirect_uri=" + "%s&client_secret=%s";

    /**
     * 开源中国获取token
     */
    public static final String OSC_AUTHORIZATION_CODE_URL = "https://www.oschina.net/action/openapi/token";

    /**
     * QQ获取用户信息
     */
    public static final String QQ_USER_INFO_URL = "https://graph.qq.com/oauth2.0/me?access_token=%s";

    /**
     * 码云获取用户信息
     */
    public static final String GITEE_USER_INFO_URL = "https://gitee.com/api/v5/user?access_token=%s";

    /**
     * 开源中国用户信息
     */
    public static final String OSC_USER_INFO_URL =
        "https://www.oschina.net/action/openapi/user?access_token=%s&dataType=json";

    /**
     * 钉钉获取 token
     */
    public static final String DING_OLD_GET_TOKEN = "https://oapi.dingtalk.com/gettoken";

    /**
     * 钉钉同步部门列表
     */
    public static final String DING_OLD_DEPT_URL = "https://oapi.dingtalk.com/topapi/v2/department/listsub";

    /**
     * 钉钉部门用户id列表
     */
    public static final String DING_DEPT_USERIDS_URL = "https://oapi.dingtalk.com/topapi/user/listid";

    /**
     * 钉钉用户详情
     */
    public static final String DING_USER_INFO_URL = "https://oapi.dingtalk.com/topapi/v2/user/get";

    /**
     * {bcrypt} 加密的特征码
     */
    public static final String BCRYPT = "{bcrypt}";

    /**
     * 客户端模式
     */
    public static final String CLIENT_CREDENTIALS = "client_credentials";

    /**
     * 客户端编号
     */
    public static final String CLIENT_ID = "client_id";

    /**
     * 客户端唯一令牌
     */
    public static final String CLIENT_RECREATE = "recreate_flag";

    /**
     * 用户ID字段
     */
    public static final String DETAILS_USER_ID = "user_id";

    /**
     * 用户名
     */
    public static final String DETAILS_USERNAME = "username";

    /**
     * 姓名
     */
    public static final String NAME = "name";

    /**
     * 协议字段
     */
    public static final String DETAILS_LICENSE = "license";

    /**
     * 激活字段 兼容外围系统接入
     */
    public static final String ACTIVE = "active";

    /**
     * AES 加密
     */
    public static final String AES = "aes";

    /**
     * 授权码模式confirm
     */
    public static final String CUSTOM_CONSENT_PAGE_URI = "/token/confirm_access";

    /**
     * {noop} 加密的特征码
     */
    public static final String NOOP = "{noop}";

    /**
     * 短信登录 参数名称
     */
    public static final String SMS_PARAMETER_NAME = "mobile";

    /**
     * 手机号登录
     */
    public static final String APP = "mobile";

    /**
     * 用户信息
     */
    public static final String DETAILS_USER = "user_info";

}
