package com.yaoyicloud.easier.common.core.util;

import org.apache.pdfbox.Loader;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;
import java.util.Map;

/**
 * 图片处理类
 */
public class ImageUtils {
    public static List<String> convertPdfToImages(String pdfUrl) throws Exception {
        URL url = new URL(pdfUrl);
        InputStream is = url.openStream();
        PDDocument document = Loader.loadPDF(is.readAllBytes());
        PDFRenderer renderer = new PDFRenderer(document);
        List<String> images = new ArrayList<>();
        for (int i = 0; i < document.getNumberOfPages(); i++) {
            BufferedImage image = renderer.renderImageWithDPI(i, 150);
            images.add(bufferedImageToBase64(image));
        }
        document.close();
        return images;
    }

    /**
     * BufferedImage转base64
     *
     * @param bufferedImage
     * @return
     */
    public static String bufferedImageToBase64(BufferedImage bufferedImage) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        try {
            // 设置图片格式
            ImageIO.write(bufferedImage, "jpg", stream);
        } catch (IOException e) {
            e.printStackTrace();
        }
        byte[] bytes = Base64.getEncoder().encode(stream.toByteArray());
        String base64 = new String(bytes);
        return base64;
    }

    public static String imgUrlToBase64(String imageUrl) {
        try {
            URL url = new URL(imageUrl);
            InputStream is = url.openStream();
            byte[] bytes = is.readAllBytes(); // 从流中读取所有字节
            is.close();
            return Base64.getEncoder().encodeToString(bytes);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static String getFileExtensionOrigin(String path) {
        if (path != null && path.contains(".")) {
            String ext = path.substring(path.lastIndexOf(".") + 1);
            if (ext.indexOf("?") > 0) {
                ext = ext.substring(0, ext.indexOf("?"));
            }
            return ext.toLowerCase();
        }
        return "";
    }

    public static String getFileExtension(String path) {
        String ext = getFileExtensionOrigin(path);
        if ("pdf".equals(ext)) {
            return "tiff";
        }
        return ext;
    }

    public static List<String> urlToBase64(String url) {
        // 常见的图片文件扩展名列表
        try {
            List<String> imageExtensions = Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "tiff", "svg");
            String ext = getFileExtensionOrigin(url);
            if (imageExtensions.contains(ext)) {
                List<String> list = new ArrayList<>();
                list.add(imgUrlToBase64(url));
                return list;
            } else if (ext.equals("pdf")) {
                return convertPdfToImages(url);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        return null;
    }

    public static List<Map<String, String>> urlToBase64Map(String url) {
        // 常见的图片文件扩展名列表
        try {
            List<String> imageExtensions = Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "tiff", "svg");
            String ext = getFileExtensionOrigin(url);
            if (imageExtensions.contains(ext)) {
                List<Map<String, String>> list = new ArrayList<>();
                list.add(imgUrlToBase64Map(url));
                return list;
            } else if (ext.equals("pdf")) {
                return convertPdfToImagesMap(url);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        return null;
    }

    public static List<Map<String, String>> convertPdfToImagesMap(String pdfUrl) throws Exception {
        URL url = new URL(pdfUrl);
        InputStream is = url.openStream();
        PDDocument document = Loader.loadPDF(is.readAllBytes());
        PDFRenderer renderer = new PDFRenderer(document);
        List<Map<String, String>> images = new ArrayList<>();
        for (int i = 0; i < document.getNumberOfPages(); i++) {
            BufferedImage image = renderer.renderImageWithDPI(i, 150);
            images.add(bufferedImageToBase64Map(image));
        }
        document.close();
        return images;
    }

    public static Map<String, String> imgUrlToBase64Map(String imageUrl) {
        double width = 456.5;
        try {
            URL url = new URL(imageUrl);
            InputStream is = url.openStream();
            byte[] bytes = is.readAllBytes(); // 从流中读取所有字节
            is.close();
            ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
            BufferedImage image = ImageIO.read(bis);
            String height = getImgHeight(image, width);
            bis.close();
            return Map.of("img", Base64.getEncoder().encodeToString(bytes), "width", String.valueOf(width), "height",
                height);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static Map<String, String> bufferedImageToBase64Map(BufferedImage bufferedImage) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        double width = 456.5;
        try {
            // 设置图片格式
            ImageIO.write(bufferedImage, "jpg", stream);
            String height = getImgHeight(bufferedImage, width);
            byte[] bytes = Base64.getEncoder().encode(stream.toByteArray());
            String base64 = new String(bytes);
            return Map.of("img", base64, "width", String.valueOf(width), "height", height);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 计算图片自适应后的高度
     *
     * @param image
     * @return
     */
    private static String getImgHeight(BufferedImage image, double width) {
        int widthInput = image.getWidth();
        int heightInput = image.getHeight();
        double height = (heightInput * width) / widthInput;
        if (height > 645) {
            height = 645;
            width = (widthInput * height) / heightInput;
        }
        DecimalFormat df = new DecimalFormat("#0.0");
        return df.format(height);
    }
}
