package com.yaoyicloud.easier.common.core.util;

import java.util.concurrent.TimeUnit;

import org.lionsoul.ip2region.xdb.Searcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Ip工具类
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2022-12-21 14:41
 */
public final class Ip2Region {

    @SuppressWarnings("checkstyle:ConstantName")
    private static final Logger log = LoggerFactory.getLogger(Ip2Region.class);

    private static byte[] cBuff;

    /**
     * 从 dbPath 加载整个 xdb 到内存。
     */
    public Ip2Region(String xdbPath) {

        try {
            cBuff = Searcher.loadContentFromFile(xdbPath);
        } catch (Exception e) {
            log.warn("failed to load content", e);
        }

    }

    /**
     * 获取IP属地
     * <p>
     * 格式为：[province]·[city]
     * </p>
     *
     * @param remoteIp 远程IP
     * @return IP属地
     */
    public String toRegion(final String remoteIp) {

        Searcher searcher;
        try {
            searcher = Searcher.newWithBuffer(cBuff);
        } catch (Exception e) {
            log.error("failed to create content cached searcher: %s\n", e);
            return null;
        }

        try {
            // 查询IP属地
            long sTime = System.nanoTime();
            String search = searcher.search(remoteIp);
            long cost = TimeUnit.NANOSECONDS.toMicros(System.nanoTime() - sTime);
            log.info("{region: {}, ioCount: {}, took: {} μs}", search, searcher.getIOCount(), cost);

            // 封装为固定格式[province]·[city]
            search = search.replace("|0", "").replace("0|", "");
            String[] split = search.split("\\|");
            return (split.length == 1) ? split[0]
                : ((split.length == 2) ? split[0] + "·" + split[1] : split[1] + "·" + split[2]);

        } catch (Exception e) {
            log.error("获取IP属地失败", e);
            throw new RuntimeException(e.getMessage());
        }
    }

}
