package com.yaoyicloud.easier.common.core.util;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 * 数学计算处理类
 */
public class MathUtils {

    /**
     * 计算工具 比例、百分比
     * @param numStr1
     * @param numStr2
     * @param isPercent
     * @return
     */
    public static String numericPercent(String numStr1, String numStr2, boolean isPercent, boolean isRise) {
        try {
            double numerator = Double.parseDouble(numStr1.replaceAll("-", "0"));
            double denominator = Double.parseDouble(numStr2.replaceAll("-", "0"));
            if (denominator == 0) {
                return "0";
            }
            double percentage;
            if (isPercent) {
                if (isRise) {
                    percentage = (numerator / denominator - 1) * 100;
                } else {
                    percentage = (numerator / denominator) * 100;
                }
            } else {
                if (isRise) {
                    percentage = (numerator / denominator - 1);
                } else {
                    percentage = (numerator / denominator);
                }
            }
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(2);
            String returnValue;
            if (isPercent) {
                returnValue = nf.format(percentage) + "%";
            } else {
                returnValue = nf.format(percentage);
            }
            return returnValue;
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return "0";
        }
    }

    /**
     * 计算工具
     * @param num1
     * @param num2
     * @param isPercent
     * @param isRise
     * @return
     */
    public static BigDecimal numericPercent(BigDecimal num1, BigDecimal num2, boolean isPercent, boolean isRise) {
        num1 = num1 != null ? num1 : new BigDecimal(0);
        num2 = num2 != null ? num2 : new BigDecimal(0);
        String result = numericPercent(String.valueOf(num1), String.valueOf(num2), isPercent, isRise);
        result = result.replaceAll("%", "").replaceAll(",", "");
        return new BigDecimal(result);
    }

    /**
     * 计算工具 计算百分比的平均值
     * @param percent1
     * @param percent2
     * @param percent3
     * @return
     */
    public static String numericPercentAvg(String percent1, String percent2, String percent3, boolean isPercent) {
        // 校验空值
        if (percent1 == null || percent2 == null || percent3 == null) {
            return "0";
        }

        // 统一转换
        double num1 = Double.parseDouble(percent1.trim().replaceAll("%", ""));
        double num2 = Double.parseDouble(percent2.trim().replaceAll("%", ""));
        double num3 = Double.parseDouble(percent3.trim().replaceAll("%", ""));

        // 计算平均值
        double average = (num1 + num2 + num3) / 3.0;

        // 格式化输出
        DecimalFormat df;
        if (isPercent) {
            df = new DecimalFormat("0.00%");
            return df.format(average / 100.0);  // 转换为百分比格式
        } else {
            df = new DecimalFormat("0.00");
            return df.format(average);  // 转换为比例
        }
    }

}
