package com.yaoyicloud.easier.common.data.application;

import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.yaoyicloud.easier.common.core.constant.CommonConstants;
import com.yaoyicloud.easier.common.data.tenant.TenantContextHolder;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;

@Slf4j
@RequiredArgsConstructor
public class EasierApplicationHandler implements TenantLineHandler {

    private final EasierApplicationConfigProperties properties;

    /**
     * 获取应用 ID 值表达式，只支持单个 ID 值
     * <p>
     * 
     * @return 应用 ID 值表达式
     */
    @Override
    public Expression getTenantId() {
        Long id = TenantContextHolder.getApplicationId();
        log.debug("当前应用为 >> {}", id);

        if (id == null) {
            return new LongValue(CommonConstants.APPLICATION_ID_1); // FXY is default
        }
        return new LongValue(id);
    }

    /**
     * 获取应用字段名
     * 
     * @return 应用字段名
     */
    @Override
    public String getTenantIdColumn() {
        return properties.getColumn();
    }

    /**
     * 根据表名判断是否忽略拼接应用条件
     * <p>
     * 
     * @param tableName 表名
     * @return 是否忽略, true:表示忽略，false:需要解析并拼接应用条件
     */
    @Override
    public boolean ignoreTable(String tableName) {
        return !properties.getTables().contains(tableName);
    }

}
