package com.yaoyicloud.easier.common.data.datascope;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.toolkit.Db;
import com.github.yulichang.base.MPJBaseMapper;

/**
 * mapper 扩展
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-12-13 21:46
 */
@SuppressWarnings("checkstyle:JavadocType")
public interface EasierBaseMapper<T> extends MPJBaseMapper<T> {

    /**
     * 根据 entity 条件，查询全部记录
     *
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     * @param scope 数据权限范围
     * @return List<T>
     */
    List<T> selectListByScope(@Param(Constants.WRAPPER) Wrapper<T> queryWrapper, DataScope scope);

    /**
     * 根据 entity 条件，查询全部记录（并翻页）
     *
     * @param page 分页查询条件（可以为 RowBounds.DEFAULT）
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     * @param scope 数据权限范围
     * @return Page
     */
    <E extends IPage<T>> E selectPageByScope(E page, @Param(Constants.WRAPPER) Wrapper<T> queryWrapper,
        DataScope scope);

    /**
     * 根据 Wrapper 条件，查询总记录数
     *
     * @param queryWrapper 实体对象封装操作类（可以为 null）
     * @param scope 数据权限范围
     * @return Integer
     */
    Long selectCountByScope(@Param(Constants.WRAPPER) Wrapper<T> queryWrapper, DataScope scope);

    /**
     * 根据ID查询(忽略逻辑查询)
     *
     * @param id 主键ID
     * @return {@link T } 实体对象
     */
    T selectRealById(Serializable id);

    /**
     * 批量保存
     *
     * @param entityList 实体列表
     * @return boolean 是否保存成功
     */
    default <T> boolean saveBatch(Collection<T> entityList) {
        return Db.saveBatch(entityList);
    }

    /**
     * 批量更新
     *
     * @param entityList 实体列表
     * @return boolean 是否更新成功
     */
    default <T> boolean updateBatchById(Collection<T> entityList) {
        return Db.updateBatchById(entityList);
    }

}
