package com.yaoyicloud.easier.common.data.tenant;

import com.alibaba.ttl.TransmittableThreadLocal;

import lombok.experimental.UtilityClass;

@UtilityClass
public class TenantContextHolder {

    @SuppressWarnings("checkstyle:MemberName")
    private final ThreadLocal<Long> THREAD_LOCAL_TENANT = new TransmittableThreadLocal<>();

    @SuppressWarnings("checkstyle:MemberName")
    private final ThreadLocal<Boolean> THREAD_LOCAL_TENANT_SKIP_FLAG = new TransmittableThreadLocal<>();

    @SuppressWarnings("checkstyle:MemberName")
    private final ThreadLocal<Long> THREAD_LOCAL_APPLICATION = new TransmittableThreadLocal<>();

    /**
     * TTL 设置租户ID<br/>
     * <b>谨慎使用此方法,避免嵌套调用。尽量使用 {@code TenantBroker} </b>
     *
     * @param tenantId 租户ID
     * @see TenantBroker
     */
    public void setTenantId(Long tenantId) {
        THREAD_LOCAL_TENANT.set(tenantId);
    }

    /**
     * 设置是否过滤的标识
     */
    public void setTenantSkip() {
        THREAD_LOCAL_TENANT_SKIP_FLAG.set(Boolean.TRUE);
    }

    /**
     * 获取TTL中的租户ID
     *
     * @return 租户ID
     */
    public Long getTenantId() {
        return THREAD_LOCAL_TENANT.get();
    }

    /**
     * 获取是否跳过租户过滤的标识
     *
     * @return 过滤结果
     */
    public Boolean getTenantSkip() {
        return THREAD_LOCAL_TENANT_SKIP_FLAG.get() != null && THREAD_LOCAL_TENANT_SKIP_FLAG.get();
    }

    public void clearTenant() {
        THREAD_LOCAL_TENANT.remove();
        THREAD_LOCAL_TENANT_SKIP_FLAG.remove();
    }

    public void setApplicationId(Long id) {
        THREAD_LOCAL_APPLICATION.set(id);
    }

    public static Long getApplicationId() {
        return THREAD_LOCAL_APPLICATION.get();
    }

    public void clearApplication() {
        THREAD_LOCAL_APPLICATION.remove();
    }

}
