package com.yaoyicloud.easier.common.data.tenant;

import javax.servlet.FilterChain;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.GenericFilterBean;

import com.yaoyicloud.easier.common.core.constant.CommonConstants;

import cn.hutool.core.util.StrUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class TenantContextHolderFilter extends GenericFilterBean {

    private static final String UNDEFINED_STR = "undefined";

    @Override
    @SneakyThrows
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) {
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;

        TenantContextHolder.setApplicationId(
            getParamId(request, CommonConstants.APPLICATION_ID_HEADER, CommonConstants.APPLICATION_ID_1));

        log.debug("获取header中的应用ID为:{}", TenantContextHolder.getApplicationId());

        TenantContextHolder
            .setTenantId(getParamId(request, CommonConstants.TENANT_ID_HEADER, CommonConstants.TENANT_ID_1));

        log.debug("获取header中的租户ID为:{}", TenantContextHolder.getTenantId());

        filterChain.doFilter(request, response);

        TenantContextHolder.clearTenant();
        TenantContextHolder.clearApplication();
    }

    private long getParamId(HttpServletRequest request, String key, long defaultValue) {
        String headerId = request.getHeader(key);
        String paramId = request.getParameter(key);

        if (StrUtil.isNotBlank(headerId) && !StrUtil.equals(UNDEFINED_STR, headerId)) {
            return Long.parseLong(headerId);
        } else if (StrUtil.isNotBlank(paramId) && !StrUtil.equals(UNDEFINED_STR, paramId)) {
            return Long.parseLong(paramId);
        } else {
            return defaultValue;
        }
    }
}
