package com.yaoyicloud.easier.common.domain;

import java.io.Serializable;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * 分页查询实体
 *
 * @author jimmy
 * @version 1.0.0
 * @date 12/19/23 12:08
 */
@SuppressWarnings("checkstyle:JavadocType")
@Data
@NoArgsConstructor(access = AccessLevel.NONE)
public abstract class EasierPaginationEntity<T> implements Serializable {

    private static final long serialVersionUID = -4928914279527370524L;

    /**
     * 当前页
     */
    @NotNull(message = "页码必填")
    @Min(value = 1L, message = "页码最小值为${value}")
    protected Long current;

    /**
     * 每页记录数
     */
    @NotNull(message = "每页记录数必填")
    @Min(value = 1L, message = "每页记录数最小值为${value}")
    @Max(value = 100L, message = "每页记录数最大值为${value}")
    protected Long size;

    /**
     * 生成分页实体
     *
     * @return 分页实体
     */
    public Page<T> getPage() {
        return new Page<>(this.current, this.size);
    }
}
