package com.yaoyicloud.easier.common.domain.common;

import java.io.Serializable;
import java.util.Map;

import com.yaoyicloud.easier.common.domain.enums.type.CommonType;

import cn.hutool.core.lang.Pair;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * 精细化评分规则项，可json序列化
 *
 */
@Data
@NoArgsConstructor
public class CommonLogicFineRuleModel implements Serializable {
    private static final long serialVersionUID = -6389402890130074848L;

    /**
     * 判断逻辑类型启用项
     */
    private String[] logicTypeEnable;

    /**
     * 字段值规则
     */
    private Map<String, ValueRule> logicToValueRuleMap;

    /**
     * 精细化评分规则项子项
     */
    private CommonLogicRuleItem[] ruleItems;

    public Pair<Boolean, Long> calculate(String val, String itemType) {
        if (ArrayUtil.isNotEmpty(ruleItems)) {
            boolean isMatch = false;
            for (CommonLogicRuleItem item : ruleItems) {
                if (StrUtil.isNotBlank(itemType) && !StrUtil.equals(itemType, item.getItemType())) {
                    // 规则项类型不匹配则跳过
                    continue;
                }
                
                Pair<Boolean, Long> result = item.calculate(val);
                if (result.getKey()) {
                    // 匹配成功，根据targetVal判断返回true还是false
                    return new Pair<Boolean, Long>(result.getValue() > 0, result.getValue());
                } else {
                    //  匹配失败，根据targetVal判断返回true还是false
                    if (result.getValue() <= 0) {
                        isMatch = true;
                    }
                }
            }

            return new Pair<Boolean, Long>(isMatch, 0L);
        }
        // score 没有意义
        return new Pair<Boolean, Long>(false, 0L);
    }

    /**
     * 值规则
     *
     * @author snows
     * @date 2025/04/25
     */
    @Data
    public static class ValueRule implements Serializable {
        private static final long serialVersionUID = 3193037987204367475L;

        /**
         * 数据类型
         */
        private String[] dataTypeEnable;

        /**
         * 单位类型
         */
        private CommonType.RuleUnitType unitType;

        /**
         * 小数点位数
         */
        private Integer decimalPlacesNumber;
    }
}
