package com.yaoyicloud.easier.common.domain.enums.state;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;

/**
 * 文件状态
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/01/12 13:34
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class FilerepoState {

    // 文件状态
    @Getter
    @AllArgsConstructor
    public enum InvoiceState {
        UNCHECK("UNCHECK", "未校验"), NORMAL("NORMAL", "正常"), ABNORMAL("ABNORMAL", "作废"), REV_STATE("REV_STATE", "红冲"),
        PART_REV_STATE("PART_REV_STATE", "部分红冲"), ALL_REV_STATE("ALL_REV_STATE", "全额红冲");

        @EnumValue
        private final String state;
        private final String description;

        /**
         * 根据发票状态返回对应枚举
         *
         * @param value 原始发票状态
         * @return {@link InvoiceState } 发票状态枚举
         */
        public static InvoiceState of(String value) {
            InvoiceState invoiceState = null;
            switch (value) {
                case "1":
                    // 未作废 1
                    invoiceState = NORMAL;
                    break;
                case "2":
                    // 作废 2
                    invoiceState = ABNORMAL;
                    break;
                case "3":
                    // 红冲 3
                    invoiceState = REV_STATE;
                    break;
                case "7":
                    // 部分红冲 7
                    invoiceState = PART_REV_STATE;
                    break;
                case "8":
                    invoiceState = ALL_REV_STATE;
                    break;
                default:
            }

            return invoiceState;
        }

    }

    // 申报状态
    @Getter
    @AllArgsConstructor
    public enum DeclarationState {

        UNREAD("UNREAD", "未读"), READ("READ", "已读"), SUBMITTED("SUBMITTED", "已提交"), CHECKED("CHECKED", "已查验"),
        ROLLBACK("ROLLBACK", "已回退");

        @EnumValue
        private final String state;
        private final String description;

    }

    // 申报状态
    @Getter
    @AllArgsConstructor
    public enum RiskInfoCheckState {

        UN_CHECK("UN_CHECK", "未审核"), TO_RECT("TO_RECT", "待整改"), REJECTED("REJECTED", "已拒绝"), PASSED("PASSED", "已通过");

        @EnumValue
        private final String state;
        private final String description;

    }

    // 过程状态
    @Getter
    @AllArgsConstructor
    public enum ProcessState {

        /* 用序数字来简单模拟单向递增状态机 */
        PRE_REFRESH(1, "待刷新"), PRE_SEND_DECL(2, "待发申报"), PRE_SUBMIT_DECL(3, "待提交资料"), PRE_CHECK(4, "待风险校验"),
        PRE_ASSERT(5, "待人工复核"), ASSERT_DONE(6, "已审核");

        @EnumValue
        private final int state;
        private final String description;

        @SuppressWarnings("checkstyle:ReturnCount")
        public static ProcessState of(Integer value) {
            switch (value) {
                case 1:
                    return PRE_REFRESH;
                case 2:
                    return PRE_SEND_DECL;
                case 3:
                    return PRE_SUBMIT_DECL;
                case 4:
                    return PRE_CHECK;
                case 5:
                    return PRE_ASSERT;
                case 6:
                    return ASSERT_DONE;
                default:
                    return null;
            }
        }

        public static ProcessState next(ProcessState curr) {
            int n = curr.state + 1;
            return ProcessState.of(n);
        }
    }
}
