package com.yaoyicloud.easier.common.domain.enums.state;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;

/**
 * 三方状态枚举
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-01-21 16:04
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class PartnershipState {

    // 认证状态
    @Getter
    @AllArgsConstructor
    public enum CertState {

        UN_CERT("UN_CERT", "未认证"), SIGNED("SIGNED", "已签约"), BOUND("BOUND", "已绑卡"), AGREEMENT("AGREEMENT", "已签协议"),
        CHECKED("CHECKED", "已合身"), CERT("CERT", "已认证");

        @EnumValue
        private final String state;

        private final String message;
    }

    // 订单状态
    @Getter
    @AllArgsConstructor
    public enum OrderState {

        PENDING("PENDING", "待支付"), WORKING("WORKING", "支付中"), FILLED("FILLED", "已付款"), FAILED("FAILED", "已失败"),
        REJECTED("REJECTED", "已驳回"), CANCELED("CANCELED", "已取消");

        @EnumValue
        private final String state;

        private final String description;
    }

    // 流水交易状态
    @Getter
    @AllArgsConstructor
    public enum TradeState {

        PENDING("PENDING", OrderState.PENDING, "待支付"), WORKING("WORKING", OrderState.WORKING, "支付中"),
        FILLED("FILLED", OrderState.FILLED, "已付款"), FAILED("FAILED", OrderState.FAILED, "已失败");

        @EnumValue
        private final String state;
        private final OrderState orderState;
        private final String description;

    }
}
