package com.yaoyicloud.easier.common.domain.enums.type;

import java.util.HashMap;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;

/**
 * CMS中心类型枚举
 *
 * @author jimmy
 * @version 1.0.2
 * @date 2022-10-15 20:14
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class FilerepoType {

    // 目标文件夹
    @Getter
    @AllArgsConstructor
    public enum TargetFolder {

        AVATAR("avatar", "头像", Boolean.TRUE), TEMPLATE("template", "模板", Boolean.TRUE),
        COVER("cover", "封面", Boolean.FALSE), PRODUCT("static/product", "商品信息", Boolean.FALSE),
        TEMPORARY("temporary", "临时文件", Boolean.TRUE), EID("eid", "核身相关内容", Boolean.TRUE),
        BANKCARD("bankcard", "银行卡片", Boolean.TRUE), GIG("gig", "Gig", Boolean.TRUE), ORDER("order", "订单", Boolean.TRUE),
        LICENSE("tenant/license", "营业执照", Boolean.TRUE), ADD_INFO("tenant/additional", "补充信息", Boolean.TRUE),
        INVOICE("invoice", "发票", Boolean.TRUE), FILE("file", "手动上传", Boolean.TRUE),
        DECLARATION("declaration", "申报内容", Boolean.TRUE), AGREEMENT("agreement", "协议", Boolean.TRUE),
        STATIC("static", "静态资源", Boolean.FALSE);

        @EnumValue
        private final String folder;
        private final String description;
        private final Boolean auth;
    }

    // OCR资源类型
    @Getter
    @AllArgsConstructor
    public enum OcrResourceType {

        BANK_CARD("BANK_CARD", "银行卡"), ID_CARD("ID_CARD", "身份证"), BIZ_LICENSE("BIZ_LICENSE", "营业执照");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 文件类型
    @Getter
    @AllArgsConstructor
    public enum FileType {

        IMG("IMG", "图片"), VIDEO("VIDEO", "视频"), AUDIO("AUDIO", "音频"), EXCEL("EXCEL", "Excel"), WORD("WORD", "Word"),
        PDF("PDF", "PDF"), PPT("PPT", "PPT"), MP4("MP4", "MP4"), MOV("MOV", "MOV"), AVI("AVI", "AVI"), TXT("TXT", "文本"),
        ATTACHMENT("ATTACHMENT", "附件");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 单位类型
    @Getter
    @AllArgsConstructor
    public enum UnitType {

        KB("KB", "KB"), GB("GB", "GB"), MB("MB", "MB");

        @EnumValue
        private final String type;
        private final String description;

    }

    // 文件组类型枚举
    @Getter
    @AllArgsConstructor
    public enum FileGroupType {
        SYS("SYS", "系统"), IMG("IMG", "图片"), DOC("DOC", "文档"), VIDEO("VIDEO", "视频");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 信息类型
    @Getter
    @AllArgsConstructor
    public enum MessageType {

        ANNOUNCEMENT("ANC", "公告"), INNER_MSG("INNER_MSG", "站内信"), EMAIL("EMAIL", "电子邮件"),
        MIME_EMAIL("MIME_EMAIL", "MIME电子邮件"), TMPL_EMAIL("TMPL_EMAIL", "TEMPLATE电子邮件"),
        TMPL_EMAIL1("TMPL_EMAIL1", "TEMPLATE电子邮件1");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 企业标识类型
    @Getter
    @AllArgsConstructor
    public enum EnterpriseIdentifierType {

        ENT_NAME("ENT_NAME", "企业名称"), REG_CODE("REG_CODE", "统一编码");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 三方类型枚举
    @Getter
    @AllArgsConstructor
    public enum PartnerType {

        QCC("QCC", "企查查"),
        NETOCR("NETOCR", "翔云OCR"),
        WFQ("WFQ", "微风企"),
        QXB("QXB", "企信宝"),
        ALLINPAY("ALLINPAY", "通联支付"),
        DECLARATION("DECLARATION", "平台申报");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 申报类型
    @Getter
    @AllArgsConstructor
    public enum DeclarationType {

        FILL("FILL", "补充申报"), SELF("SELF", "自主申报");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 申报操作方法
    @Getter
    @AllArgsConstructor
    public enum DeclarationOperateMethod {

        CREATE("CREATE", "创建"), OPEN("OPEN", "打开"), SUBMIT("SUBMIT", "提交"), CHECK("CHECK", "检验");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 课程类型
    @Getter
    @AllArgsConstructor
    public enum LessonType {

        ARTICLE("ARTICLE", "文章"), AUDIO("AUDIO", "音频"), VIDEO("VIDEO", "视频");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 课程模式类型
    @Getter
    @AllArgsConstructor
    public enum LessonPatternType {
        TEXT("TEXT", LessonType.ARTICLE, "文本"), GRAPHICS_TEXT("GRAPHICS_TEXT", LessonType.ARTICLE, "图文"),
        AUDIO("AUDIO", LessonType.AUDIO, "音频"), VIDEO("VIDEO", LessonType.VIDEO, "视频");

        @EnumValue
        private final String type;
        private final LessonType lessonType;
        private final String description;
    }

    // 设置类型
    @Getter
    @AllArgsConstructor
    public enum SettingType {

        CONTENT_PROTECT("CONTENT_PROTECT", "内容保护"), CONTENT_ACTION("CONTENT_ACTION", "内容活动"),
        BIZ_PROMOTION("BIZ_PROMOTION", "商业推广");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 风险项校验类型
    @Getter
    @AllArgsConstructor
    public enum EntRiskItemCheckType {

        BY_LOWER("BY_LOWER", "法审项"), BY_ENT("BY_ENT", "企审项"), BY_STATISTICS("BY_STATISTICS", "统计项");

        @EnumValue
        private final String type;
        private final String description;
    }

    /**
     * 风险项实际值类型
     *
     * @author snows
     * @date 2025/01/08
     */
    @Getter
    @AllArgsConstructor
    public enum EntRiskItemActualValueType {
        TEXT("TEXT", "文本"), TABLE("TABLE", "表格"), PDF("PDF", "PDF"), IMG("IMG", "图片");

        @EnumValue
        private final String type;
        private final String description;
    }

    /**
     * 填写形式类型
     *
     * @author snows
     * @date 2025/03/27
     */
    @Getter
    @AllArgsConstructor
    public enum FillFormType {
        EXCEL("EXCEL", "Excel"), SYS_FORM("SYS_FORM", "系统表单"), OCR("OCR", "OCR识别");

        @EnumValue
        private final String type;
        private final String description;
    }

    /**
     * 文件解析器类型
     *
     * @author snows
     * @date 2025/03/21
     */
    @Getter
    @AllArgsConstructor
    public enum FileParserType {

        COMMON_FINANCIAL_EXCEL_PARSER("COMMON_FINANCIAL_EXCEL_PARSER", "平台公司/推广服务商等普通企业财务信息Excel解析器", "科目",
            getCommonFinancialTemplateMap(), getCommonFinancialIndexMap()),
        FOUNDATION_EXCEL_PARSER("FOUNDATION_EXCEL_PARSER", "基金会财务信息Excel解析器", "内容", getFoundationTemplateMap(),
            getFoundationIndexMap());

        /**
         * 类型
         */
        private final String type;

        /**
         * 描述
         */
        private final String desc;

        /**
         * 年份的行标记
         */
        private final String yearRowMark;

        /**
         * 模板map
         */
        private final Map<String, String> templateMap;

        /**
         * 索引map
         */
        private final Map<Integer, String> indexMap;

        /**
         * 平台公司/推广服务商等普通企业财务信息Excel解析map
         *
         * @return {@link Map }<{@link String }, {@link String }> 模板map
         */
        public static Map<String, String> getCommonFinancialTemplateMap() {
            final Map<String, String> itemMap = new HashMap<>(20);
            itemMap.put("流动资产合计", "flowAssets");
            itemMap.put("流动负债合计", "flowLiabilities");
            itemMap.put("营运资本", "workingCapital");
            itemMap.put("固定资产", "fixedAsset");
            itemMap.put("存货", "inventory");
            itemMap.put("资产合计", "totalAssets");
            itemMap.put("应收账款", "receivables");
            itemMap.put("负债合计", "liabTotal");
            itemMap.put("所有者权益", "las3yTotAmtLiaEquMap");
            itemMap.put("营业收入", "operatingIncome");
            itemMap.put("主营业务收入", "mainBusInc");
            itemMap.put("营业利润", "mainBusProfit");
            itemMap.put("净利润", "netProfit");
            itemMap.put("利润总额", "las3yTotProfMap");
            itemMap.put("利息支出", "interestExpense");
            return itemMap;
        }

        /**
         * 平台公司/推广服务商等普通企业财务信息Excel解析map
         *
         * @return {@link Map }<{@link String }, {@link String }> 索引map
         */
        public static Map<Integer, String> getCommonFinancialIndexMap() {
            final Map<Integer, String> itemMap = new HashMap<>(20);
            itemMap.put(0, "科目");
            itemMap.put(2, "流动资产合计");
            itemMap.put(3, "流动负债合计");
            itemMap.put(4, "营运资本");
            itemMap.put(5, "固定资产");
            itemMap.put(6, "存货");
            itemMap.put(7, "资产合计");
            itemMap.put(8, "应收账款");
            itemMap.put(9, "负债合计");
            itemMap.put(10, "所有者权益");
            itemMap.put(11, "营业收入");
            itemMap.put(12, "主营业务收入");
            itemMap.put(13, "营业利润");
            itemMap.put(14, "净利润");
            itemMap.put(15, "利润总额");
            itemMap.put(16, "利息支出");
            return itemMap;
        }

        /**
         * 基金会财务信息Excel解析map
         *
         * @return {@link Map }<{@link String }, {@link String }> 模板map
         */
        public static Map<String, String> getFoundationTemplateMap() {
            final Map<String, String> itemMap = new HashMap<>(20);
            itemMap.put("捐赠收入", "donationRevenueScale");
            itemMap.put("捐赠项目成本", "flowLiabilities");
            itemMap.put("资产合计", "totalAssets");
            itemMap.put("净资产合计", "totalNetAssets");
            itemMap.put("收入合计", "totalRevenue");
            itemMap.put("投资收益", "investmentIncome");
            itemMap.put("政府补助收入", "governmentSubsidyIncome");
            itemMap.put("提供服务收入", "servicesRevenue");
            itemMap.put("费用合计", "totalCosts");
            itemMap.put("工资福利支出", "salaryAndWelfareExpenses");
            itemMap.put("行政办公支出", "administrativeOfficeExpenses");
            itemMap.put("业务活动成本", "businessActivitiesCost");
            itemMap.put("管理费用", "managementFees");
            itemMap.put("筹资费用", "financingCosts");
            itemMap.put("上年基金余额", "fundBalance");
            return itemMap;
        }

        /**
         * 基金会财务信息Excel解析map
         *
         * @return {@link Map }<{@link String }, {@link String }> 索引map
         */
        public static Map<Integer, String> getFoundationIndexMap() {
            final Map<Integer, String> itemMap = new HashMap<>(20);
            itemMap.put(1, "内容");
            itemMap.put(3, "捐赠收入");
            itemMap.put(4, "捐赠项目成本");
            itemMap.put(5, "资产合计");
            itemMap.put(6, "净资产合计");
            itemMap.put(7, "收入合计");
            itemMap.put(8, "投资收益");
            itemMap.put(9, "政府补助收入");
            itemMap.put(10, "提供服务收入");
            itemMap.put(11, "费用合计");
            itemMap.put(12, "工资福利支出");
            itemMap.put(13, "行政办公支出");
            itemMap.put(14, "业务活动成本");
            itemMap.put(15, "管理费用");
            itemMap.put(16, "筹资费用");
            itemMap.put(17, "上年基金余额");
            return itemMap;
        }
    }

}
