package com.yaoyicloud.easier.common.domain.enums.type;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;

/**
 * 三方类型
 *
 * @author jimmy
 * @version 1.0.2
 * @date 2022-09-06 18:30
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class PartnershipType {

    // 服务商类型
    @Getter
    @AllArgsConstructor
    public enum Merchant {
        FREERR_PAY("FREERR_PAY", "富瑞宝"), WECHAT_PAY("WECHAT_PAY", "微信支付"), ALI_PAY("ALI_PAY", "支付宝"),
        CHENG_XIN_YUN("CHENG_XIN_YUN", "橙薪云"), PAY_ACCOUNT("PAY_ACCOUNT", "薪账户"),
        HUI_TONG_ZHI_YUN("HUI_TONG_ZHI_YUN", "惠通智云");

        @EnumValue
        private final String type;
        private final String description;
    }

    // 税源地
    @Getter
    @AllArgsConstructor
    public enum TaxationCompany {
        WECHAT_PAY("WECHAT_PAY", Merchant.WECHAT_PAY, "微信支付"), ALI_PAY("ALI_PAY", Merchant.ALI_PAY, "支付宝"),
        HAO_ZHAO("HAO_ZHAO", Merchant.CHENG_XIN_YUN, "湖南好招"),
        CHENG_XIN_YUN("CHENG_XIN_YUN", Merchant.CHENG_XIN_YUN, "周口橙心云"),
        LING_QI_FU("LING_QI_FU", Merchant.CHENG_XIN_YUN, "安徽灵企服"),
        FEN_XIANG_WEI_LAI("FEN_XIANG_WEI_LAI", Merchant.HUI_TONG_ZHI_YUN, "江西纷享未来");

        @EnumValue
        private final String company;
        private final Merchant merchant;
        private final String description;
    }

    // 开票类目类型
    @Getter
    @AllArgsConstructor
    public enum InvoiceCategoryType {
        SERVICE("SERV", "服务类型"), CATEGORY("CATE", "开票类目");

        @EnumValue
        private final String type;
        private final String description;
    }

    /**
     * 操作对象类型
     */
    @Getter
    @AllArgsConstructor
    public enum TargetType {

        MERCHANT("MERCHANT", "合作商"), TAXATION_COMPANY("TAXATION_COMPANY", "税收公司");

        private final String type;
        private final String description;
    }
}
