package com.yaoyicloud.controller;


import com.yaoyicloud.annotation.EasierLog;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.constant.FilerepoCacheConstants;
import com.yaoyicloud.dto.ReportDTO;
import com.yaoyicloud.constant.enums.FilerepoErrorCodes;
import com.yaoyicloud.exception.FilerepoBizException;
import com.yaoyicloud.service.ReportService;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;


import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;


/**
 * 报告控制器
 *
 * @author snows
 * @date 2024/10/14
 */
@RestController
@RequiredArgsConstructor
@Slf4j
public class ReportController {
    private final ReportService reportService;
    private final RedisTemplate<String, Object> redisTemplate;
    private final FilerepoProperties filerepoProperties;

    /**
     * 创建Plus版本审核报告
     *
     * @param resource 请求参数
     * @return {@link Boolean } 结果
     */
    @EasierLog("创建Plus版本审核报告")
    @PostMapping("/report/create-report")
    public String
        createPlusVersionCheckReport(@Validated @RequestBody ReportDTO.OnCreatePlusVersionReport resource) throws IOException {
        synchronized (this) {
            // 设置限流
            String key = FilerepoCacheConstants.ENT_RISK_PLUS_CHECK_REPORT_RATE_LIMITING_KEY;
            Long increment = redisTemplate.opsForValue().increment(key);
            if (increment != null && increment.compareTo(filerepoProperties.getPlusReportRateLimiting() + 1) > 0) {
                redisTemplate.opsForValue().decrement(key);
                throw new FilerepoBizException(FilerepoErrorCodes.CMS_REPORT_RATE_LIMIT_ERROR);
            }

            return  reportService.createPlusVersionCheckReport(resource.getReportType(), resource.getData(),
                    resource.getReportTempWordFile(), resource.getReportBastPath());
        }
    }
}
