package com.yaoyicloud.mapper;

import java.time.LocalDateTime;
import java.util.List;

import com.yaoyicloud.constant.enums.FinancialTaxVersionType;
import com.yaoyicloud.entity.CmsRecharge;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;


/**
 * 充值记录mapper
 *
 * @author snows
 * @date 2024/09/09
 */
@Mapper
public interface CmsRechargeMapper extends BaseMapper<CmsRecharge> {

    /**
     * 统计充值记录数
     *
     * @param productVersion 产品版本
     * @param tenantId 租户ID
     * @return long 充值记录数
     */
    @InterceptorIgnore(tenantLine = "true")
    @Select("select count(*) from cms_recharge where product_version = #{productVersion} and tenant_id=#{tenantId}")
    long countRecharge(@Param("productVersion") String productVersion, @Param("tenantId") Long tenantId);

    /**
     * 查询是否购买PLUS版
     *
     * @return boolean 是否购买PLUS版
     */
    default boolean hasPlus() {
        LocalDateTime now = LocalDateTime.now();

        // 查询是否购买PLUS版
        long rechargeCount = this.selectCount(Wrappers.<CmsRecharge>lambdaQuery().le(CmsRecharge::getIssueTime, now)
            .ge(CmsRecharge::getExpireTime, now).eq(CmsRecharge::getProductVersion, FinancialTaxVersionType.PLUS));

        return rechargeCount > 0;
    }

    /**
     * 列出有效的充值记录
     *
     * @param tenantId 租户 ID
     * @return {@link List }<{@link CmsRecharge }> 有效的充值记录列表
     */
    default List<CmsRecharge> listValidRecharge(Long tenantId) {
        LocalDateTime now = LocalDateTime.now();

        return this.selectList(Wrappers.<CmsRecharge>lambdaQuery().eq(CmsRecharge::getTenantId, tenantId)
            .le(CmsRecharge::getIssueTime, now).ge(CmsRecharge::getExpireTime, now)
            .orderByAsc(CmsRecharge::getCreatedTime));
    }
}
