package com.yaoyicloud.template;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.constant.enums.ReportType;
import org.springframework.stereotype.Component;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.HackLoopTableRenderPolicy;
import com.deepoove.poi.policy.RenderPolicy;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
@Slf4j
@Component
@RequiredArgsConstructor
public class PlatformCompanyReport extends AbstractReportExporter {


    private final FilerepoProperties filerepoProperties;

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }



    @Override
    protected String getTemplate(ReportType reportType, String basePath) {
        String reportTemplateFileName = getReportTemplateFileName(reportType);
        return basePath + File.separator + reportTemplateFileName;
    }

    @Override
    protected void bindConfigure(ConfigureBuilder builder) {
        RenderPolicy scoreRenderPolicy = this.getScoreRenderPolicy();
        HackLoopTableRenderPolicy hackLoopTableRenderPolicy = new HackLoopTableRenderPolicy();
        builder.bind("checkItemScores", scoreRenderPolicy).bind("basicInfoChecks", hackLoopTableRenderPolicy)
            .bind("businessAbnormals", hackLoopTableRenderPolicy).bind("dishonestPersons", hackLoopTableRenderPolicy)
            .bind("penaltyRecords", hackLoopTableRenderPolicy).bind("taxPenalties", hackLoopTableRenderPolicy)
            .bind("severeViolations", hackLoopTableRenderPolicy).bind("questionnaireItems", hackLoopTableRenderPolicy)

            .bind("indicators", hackLoopTableRenderPolicy);
    }

    @Override
    protected String imagePath() {
        return filerepoProperties.getReportImagePath() + "/coverBackageimage.jpeg";
    }




    @Override
    protected void renderTemplate(String reportTempWordFile, String templatePath, ConfigureBuilder builder,
        Map<String, Object> data) throws IOException {
        File file = new File(reportTempWordFile);

        Configure config = builder.build();
        XWPFTemplate template = XWPFTemplate.compile(templatePath, config).render(data);
    template.writeToFile(reportTempWordFile);
        template.close();

    }



    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getReportTemplatePath();
    }

    /**
     * 获取模板文件路径
     *
     * @param reportType
     * @return
     */
    public String getReportTemplateFileName(ReportType reportType) {
        return reportType.getType().toLowerCase() + "_template.docx";
    }

    @Override
    protected List<String> getFontPaths() {
        return Arrays.asList(filerepoProperties.getSourceHanSansCnBoldFontPath(),
            filerepoProperties.getSourceHanSansCnFontPath());
    }

}
